/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.uwo.csd.nlp;

import ca.uwo.csd.nlp.cmplx.*;
import ca.uwo.csd.nlp.pred.*;
import java.util.ArrayList;

/**
 *
 * @author Syeed Ibn Faiz
 */
public class DRSTranslator {

    Atom head;

    public ArrayList<ArrayList<Atom>> translate(DRS drs) throws Exception {
        Question q = (Question) drs.getConditionByType("QUES");
        if (q == null) {
            throw new Exception("No question in DRS.");
        }

        Atom headAtom = getHeadAtom(q);
        head = headAtom; //needed by getBodyAtom
        //System.out.println("headAtom = " + headAtom);

        ArrayList<ArrayList<Atom>> bodyList = new ArrayList<ArrayList<Atom>>();
        getBodies(q.getDrs(), new ArrayList<Predicate>(), bodyList, false);
        for (ArrayList<Atom> l : bodyList) {
            /*for (Atom a : l) {
                System.out.print(a + " ");
            }
            System.out.println("");*/
            l.add(0, headAtom);
        }
        return bodyList;
    }

    private Atom getHeadAtom(Question q) {
        DRS drs = q.getNestedDRS();
        Atom headAtom = new Atom(false);
        ArrayList<Predicate> queryList = drs.getPredicatesByType("QRY");
        if (queryList == null || queryList.isEmpty()) {
            //do does question
            headAtom.appendName("wh");
            headAtom.add("yes");
            return headAtom;
        }

        Query query = (Query) queryList.get(0);
        headAtom.appendName(query.getqWord());

        if (query.getqWord().equalsIgnoreCase("which")) {
            headAtom.appendName(getNObjectType(drs, query.getRef()));
            headAtom.add(query.getRef());

        } else {
            headAtom.appendName("be");
            ArrayList<Predicate> verbList = drs.getAllPredicatesByType("VRB");
            for (Predicate p : verbList) {
                Verb v = (Verb) p;
                if (v.getVerb().equalsIgnoreCase("be") && v.getSubRef().equalsIgnoreCase(query.getRef())) {
                    headAtom.add(v.getObjRef());
                    v.setMarked(true);
                    headAtom.appendName(getNObjectType(drs, v.getObjRef()));
                    break;
                }
            }            
        }
        return headAtom;
    }

    private void getBodies(DRS drs, ArrayList<Predicate> predicateList, ArrayList<ArrayList<Atom>> bodyList, boolean negated) throws Exception {
        Condition extension = null;
        ArrayList<Condition> condList = drs.getConditions();
        for (Condition c : condList) {
            if (c instanceof CmplxStruct) {
                extension = c;
            } else {                                
                Predicate p = (Predicate) c;
                if (p.getType().equalsIgnoreCase("VRB")) {
                    ((Verb) p).setNegated(negated);
                } else if (p.getType().equalsIgnoreCase("REL")) {
                    ((Relation) p).setNegated(negated);
                }
                predicateList.add(p);
            }
        }

        if (extension == null) {                                //no more nested DRS
            ArrayList<Atom> atomList = new ArrayList<Atom>();
            for (Predicate p : predicateList) {
                if (p.getType().equalsIgnoreCase("VRB")) {
                    Verb v = (Verb) p;
                    if (!v.isMarked()) {
                        //System.out.println("Processing verb: " + v);
                        Atom atom = new Atom(v.isNegated());

                        processObject(v.getSubRef(), atom, predicateList);
                        atom.appendName(v.getVerb());

                        if (v.getObjRef() != null) {
                            Predicate prop = findPredicateByRefAndType(v.getObjRef(), "PROP", predicateList);
                            if (prop == null) {
                                processObject(v.getObjRef(), atom, predicateList);
                            } else {
                                atom.appendName(((Property) prop).getAdj());
                            }
                        }
                        

                        Predicate mod = findPredicateByRefAndType(v.getRef(), "MADV", predicateList);
                        if (mod != null) {
                            atom.appendName(((ModifierAdv) mod).getAdverb());
                        }
                        
                        mod = findPredicateByRefAndType(v.getRef(), "MPP", predicateList);
                        if (mod != null) {                            
                            atom.appendName(((ModifierPP) mod).getPrep());
                            processObject(((ModifierPP) mod).getRef2(), atom, predicateList);
                        }

                        //System.out.println("Verb processed: " + atom);
                        atomList.add(atom);

                    } 
                } else if (p.getType().equalsIgnoreCase("REL")) {
                    Relation r = (Relation) p;
                    //System.out.println("Processing rel: " + r);
                    Atom atom = new Atom(r.isNegated());
                    processObject(r.getRef1(), atom, predicateList);
                    atom.appendName("of");
                    processObject(r.getRef2(), atom, predicateList);
                    //System.out.println("Rel processed: " + atom);
                    atomList.add(atom);
                }
            }
            bodyList.add(atomList);
        } else {
            if (extension.getType().equalsIgnoreCase("DISJ")) {
                ArrayList<Predicate> predicateList2 = new ArrayList<Predicate>(predicateList);
                //saving info
                boolean negInfo[] = new boolean[predicateList.size()];
                for (int i = 0; i < predicateList.size(); i++) {
                    Predicate p = predicateList.get(i);
                    if (p.getType().equalsIgnoreCase("VRB")) {
                        negInfo[i] = ((Verb)p).isNegated();
                    } else if (p.getType().equalsIgnoreCase("REL")) {
                        negInfo[i] = ((Relation)p).isNegated();
                    }
                }
                getBodies(((Disjunction)extension).getDrs1(), predicateList, bodyList, negated);
                //restoring info
                for (int i = 0; i < predicateList2.size(); i++) {
                    Predicate p = predicateList2.get(i);
                    if (p.getType().equalsIgnoreCase("VRB")) {
                        ((Verb)p).setNegated(negInfo[i]);
                    } else if (p.getType().equalsIgnoreCase("REL")) {
                        ((Relation)p).setNegated(negInfo[i]);
                    }
                }
                getBodies(((Disjunction)extension).getDrs2(), predicateList2, bodyList, negated);
            } else if (extension.getType().equalsIgnoreCase("NEG")) {
                getBodies(((Negation)extension).getDrs(), predicateList, bodyList, negated ^ true);
            }
        }
    }

    private void processObject(String ref, Atom atom, ArrayList<Predicate> predicateList) throws Exception {
        if (ref.startsWith("named")) {
            ref = ref.substring(ref.indexOf('(') + 2, ref.indexOf(')') - 1);
            String parts[] = ref.split("_");
            if (parts.length == 1) {
                atom.add(parts[0].toLowerCase());
            } else {
                atom.appendName(parts[0].toLowerCase());
                //atom.add(parts[1].toLowerCase());
                String param = "";
                for (int i = 1; i < parts.length; i++) {
                    if (i > 1) {
                        param += "_";
                    }
                    param += parts[i];
                }
                atom.add(param.toLowerCase());
            }
        } else {
            //System.out.println("Here");
            NObject ob = (NObject) findPredicateByRefAndType(ref, "NOBJ", predicateList);
            if (ob == null) {
                String type = head.getName();
                type = type.substring(type.lastIndexOf('_') + 1);
                String headRef = head.getParam(0);
                if (headRef == null) {
                    throw new Exception("Error reading head atom from getbodies");
                } else {
                    atom.appendName(type);
                    atom.add(headRef);
                }
            } else {
                atom.appendName(ob.getNoun());
                atom.add(ref);
            }
        }
    }

    private Predicate findPredicateByRef(String ref, ArrayList<Predicate> predicateList) {
        for (Predicate p : predicateList) {
            if (p.getRef().equalsIgnoreCase(ref)) {
                return p;
            }
        }
        return null;
    }

    private Predicate findPredicateByRefAndType(String ref, String type, ArrayList<Predicate> predicateList) {
        for (Predicate p : predicateList) {
            if (p.getRef().equalsIgnoreCase(ref) && p.getType().equalsIgnoreCase(type)) {
                return p;
            }
        }
        return null;
    }

    private String getNObjectType(DRS drs, String ref) {
        ArrayList<Predicate> nObjList = drs.getAllPredicatesByRef(ref);
        if (nObjList == null) {
            return null;
        }
        for (Predicate p : nObjList) {
            if (p.getType().equals("NOBJ")) {
                return ((NObject) p).getNoun();
            }
        }
        return null;
    }
}
