/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ca.uwo.csd.nlp;

import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * Domain represents a domain of discourse. It includes a list of referents.
 * @author Syeed Ibn Faiz
 */
public class Domain {
    private ArrayList<String> referents;

    /**
     * Constructs an empty domain of discourse.
     */
    public Domain() {
        referents = null;
    }

    /**
     * Constructs a domain of discourse from a given list of strings.
     * @param referents list of referents
     */
    public Domain(ArrayList<String> referents) {
        this.referents = referents;
    }

    /**
     * Constructs a domain of discourse from a string of tokens separated by comma.
     * @param rawStr
     */
    public Domain(String rawStr) {
        this.referents = new ArrayList<String>();
        StringTokenizer tokenizer = new StringTokenizer(rawStr, ",");

        while (tokenizer.hasMoreTokens()) {
            referents.add(tokenizer.nextToken());
        }
    }

    /**
     * Tests whether a referent belongs to this domain of discourse or not.
     * @param ref a referent string
     * @return <code>true</code> if the argument belongs to this domain
     */
    public boolean isReferent(String ref) {
        return referents.contains(ref);
    }
    
    /**
     * Returns a <code>String</code> representation of this DRS.
     * @return
     */
    @Override
    public String toString() {
        if (referents == null) return "[]";
        StringBuilder builder = new StringBuilder("[");
        for (int i = 0; i < referents.size(); i++)  {
            if (i != 0) builder.append(",");
            builder.append(referents.get(i));
        }
        builder.append("]");
        return builder.toString();
    }
}
