/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ca.uwo.csd.nlp;

import ca.uwo.csd.nlp.cmplx.CmplxStruct;
import ca.uwo.csd.nlp.cmplx.Disjunction;
import ca.uwo.csd.nlp.cmplx.Negation;
import ca.uwo.csd.nlp.cmplx.Question;
import ca.uwo.csd.nlp.pred.ModifierAdv;
import ca.uwo.csd.nlp.pred.ModifierPP;
import ca.uwo.csd.nlp.pred.NObject;
import ca.uwo.csd.nlp.pred.Predicate;
import ca.uwo.csd.nlp.pred.Property;
import ca.uwo.csd.nlp.pred.Query;
import ca.uwo.csd.nlp.pred.Relation;
import ca.uwo.csd.nlp.pred.Verb;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 *
 * @author tonatuni
 */
public class QParser implements Parser {

    public DRS parse(String ACEText) throws Exception {
        //System.out.println("Parsing ACEText: " + ACEText);
        if (!ACEText.startsWith("drs")) {
            throw new Exception("APE failed to parse the given query.");
            //return null;
        }
        int i = find(ACEText, ',', 4);
        //System.out.println("i = " + i);
        Domain d = parseDomain(ACEText.substring(4, i));
        ArrayList<Condition> conditions = parseConditions(ACEText.substring(i + 1, ACEText.length() - 1));
        return new DRS(d, conditions);
    }

    private ArrayList<Condition> parseConditions(String s) throws Exception {
        //System.out.println("Parsing conditions: " + s);
        ArrayList<Condition> conditions = new ArrayList<Condition>();
        if (s.equals("[]")) return conditions;

        ArrayList<String> tokens = new ArrayList<String>();
        s = s.substring(1, s.length() - 1);     //getting rid of [ & ]
        int startIndex = 0;
        int i = -1;
        while ((i = find(s, ',', startIndex)) != -1) {
            tokens.add(s.substring(startIndex, i));
            startIndex = i + 1;
        }
        tokens.add(s.substring(startIndex));

        for (int j = 0; j < tokens.size(); j++) {
            if (tokens.get(j).startsWith("query")) conditions.add(parsePredicate("QRY", tokens.get(j)));
            else if(tokens.get(j).startsWith("object")) conditions.add(parsePredicate("NOBJ", tokens.get(j)));
            else if(tokens.get(j).startsWith("property")) conditions.add(parsePredicate("PROP", tokens.get(j)));
            else if(tokens.get(j).startsWith("relation")) conditions.add(parsePredicate("REL", tokens.get(j)));
            else if(tokens.get(j).startsWith("modifier_adv")) conditions.add(parsePredicate("MADV", tokens.get(j)));
            else if(tokens.get(j).startsWith("modifier_pp")) conditions.add(parsePredicate("MPP", tokens.get(j)));
            else if(tokens.get(j).startsWith("predicate")) conditions.add(parsePredicate("VRB", tokens.get(j)));
            else if(tokens.get(j).startsWith("question")) conditions.add(parseCmplxStruct("QUES", tokens.get(j)));
            else if(tokens.get(j).startsWith("-")) conditions.add(parseCmplxStruct("NEG", tokens.get(j)));
            else if(tokens.get(j).startsWith("v")) conditions.add(parseCmplxStruct("DISJ", tokens.get(j)));
        }
        
        return conditions;
    }

    private Predicate parsePredicate(String type, String s) {
        //System.out.println("Parsing predicate: " + s);
        s = s.substring(s.indexOf('(') + 1, s.lastIndexOf(')')); //taking the part inside of ( & )
        
        StringTokenizer tokenizer = new StringTokenizer(s, ",");
        ArrayList<String> tokens = new ArrayList<String>();
        while (tokenizer.hasMoreTokens()) {
            tokens.add(tokenizer.nextToken());
        }
        if (type.equals("QRY")) return new Query(tokens);
        else if(type.equals("NOBJ")) return new NObject(tokens);
        else if(type.equals("PROP")) return new Property(tokens);
        else if(type.equals("REL")) return new Relation(tokens);
        else if(type.equals("MADV")) return new ModifierAdv(tokens);
        else if(type.equals("MPP")) return new ModifierPP(tokens);
        else if(type.equals("VRB")) return new Verb(tokens);
        else return null;
    }

    private CmplxStruct parseCmplxStruct(String type, String s) throws Exception {
        //System.out.println("Parsing cmplxstruct: " + s);
        s = s.substring(s.indexOf('(') + 1, s.lastIndexOf(')')); //taking the part inside of ( & )
        if (type.equals("QUES")) return new Question(parse(s));
        else if(type.equals("NEG")) return new Negation(parse(s));
        else if(type.equals("DISJ")) {
            int i = find(s, ',', 0);
            return new Disjunction(parse(s.substring(0, i)), parse(s.substring(i + 1)));
        } else return null;
    }
    private Domain parseDomain(String s) {
        //System.out.println("Parsing domain: " + s);
        if (s.equals("[]")) return new Domain();
        return new Domain(s.substring(1, s.length() - 1));
    }
    /**
     * Find a character in the top level, outside of any bracket
     * @param str
     * @param ch
     */
    private int find(String str, char ch, int startIndex) {
        int b1Count = 0;
        int b2Count = 0;
        for (int i = startIndex; i < str.length(); i++) {
            if (str.charAt(i) == '(') b1Count++;
            else if (str.charAt(i) == ')') b1Count--;
            else if (str.charAt(i) == '[') b2Count++;
            else if (str.charAt(i) == ']') b2Count--;
            else if (b1Count == 0 && b2Count == 0 && str.charAt(i) == ch) return i;
        }
        return -1;
    }

}
