/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ca.uwo.csd.nlp.pred;

import java.util.ArrayList;


/**
 * The Noun-object-predicates stand for objects that are
 * introduced by the different forms of noun.
 * @author Syeed Ibn Faiz
 */
public class NObject extends Predicate {
    String ref;
    String noun;
    String classType;
    String unit;
    String op;
    String count;

    /**
     * Constructs a noun object. The object-predicates stand for objects that are
     * introduced by the different forms of noun.
     * @param ref  The variable that stands for this object and that is used for references
     * @param noun The noun (mass or countable) that was used to introduce the object.
     * @param classType This is one of {dom,mass,countable} and deﬁnes the classiﬁcation
     * of the object. The tree structure below shows the hierarchy of these values.
     * @param unit If the object was introduced together with a measurement noun (e.g. “2
     * kg of apples”) then this entry contains the value of the measurement noun (e.g. kg).
     * Otherwise, this entry is na.
     * @param op One of {eq,geq,greater,leq,less,exactly,na}. eq stands for “equal”, geq
     * for “greater or equal”, and leq for “less or equal”.
     * @param A positive number or na. Together with Unit and Op, this deﬁnes the cardinality
     * or extent of the object.
     */
    public NObject(String ref, String noun, String classType, String unit, String op, String count) {
        this.ref = ref;
        this.noun = noun;
        this.classType = classType;
        this.unit = unit;
        this.op = op;
        this.count = count;
    }
    public NObject(ArrayList<String> l) {
        this.ref = l.get(0);
        this.noun = l.get(1);
        this.classType = l.get(2);
        this.unit = l.get(3);
        this.op = l.get(4);
        this.count = l.get(5);
    }

    
    @Override
    public String getRef() {
        return ref;
    }

    @Override
    public String toString() {
        return "object("+ref+","+noun+","+classType+","+unit+","+op+","+count+")";
    }

    public String getClassType() {
        return classType;
    }

    public String getCount() {
        return count;
    }

    public String getNoun() {
        return noun;
    }

    public String getOp() {
        return op;
    }

    public String getUnit() {
        return unit;
    }

    @Override
    public String getType() {
        return "NOBJ";
    }



}
