/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package ca.uwo.csd.nlp.pred;

import java.util.ArrayList;

/**
 * The property-predicates stand for properties that are introduced by adjectives. The references
 * can either be variables or expressions. See section 3.6 for the representation of expressions.
 * @author tonatuni
 */
public class Property extends Predicate{

    String ref1;
    String ref2;
    String ref3;
    String adj;
    String degree;
    String compTarget;

    /**
     * Constructs an empty property predicate
     */
    private Property() {
        this.ref1 = null;
        this.ref2 = null;
        this.ref3 = null;
        this.adj = null;
        this.degree = null;
        this.compTarget = null;
    }

    /**
     * Constructs a unary property predicate.
     * @param ref1  The variable or expression that stands for the primary object of the property (i.e. the subject)
     * @param adj The intransitive or transitive adjective.
     * @param degree  This is one of {pos, pos as,comp,comp than,sup} and it defines the degree of the adjective
     */
    public Property(String ref1, String adj, String degree) {
        this();
        this.ref1 = ref1;
        this.adj = adj;
        this.degree = degree;        
    }
    /**
     * Constructs a binary property predicate.
     * @param ref1 The variable or expression that stands for the primary object of the property (i.e. the subject)
     * @param adj The intransitive or transitive adjective.
     * @param degree  This is one of {pos,pos as,comp,comp than,sup} and it defines the degree of the adjective
     * @param ref2  The variable or expression that stands for the secondary object of the property
     */
    public Property(String ref1, String adj, String degree, String ref2) {
        this();
        this.ref1 = ref1;
        this.ref2 = ref2;        
        this.adj = adj;
        this.degree = degree;        
    }

    /**
     * Constructs a ternary property predicate.
     * @param ref1 The variable or expression that stands for the primary object of the property (i.e. the subject)
     * @param adj The intransitive or transitive adjective.
     * @param ref2  The variable or expression that stands for the secondary object of the property
     * @param degree  This is one of {pos,pos as,comp,comp than,sup} and it defines the degree of the adjective
     * @param compTarget This is one of {subj,obj} and it defines for transitive adjectives whether the comparison targets the subject
     * @param ref3 The variable or expression that stands for the tertiary object of the property
     */
    public Property(String ref1, String adj, String ref2, String degree, String compTarget, String ref3) {
        this.ref1 = ref1;
        this.ref2 = ref2;
        this.ref3 = ref3;
        this.adj = adj;
        this.degree = degree;
        this.compTarget = compTarget;
    }
    public Property(ArrayList<String> l) {
        this();
        if (l.size() == 3) {
            this.ref1 = l.get(0);
            this.adj = l.get(1);
            this.degree = l.get(2);
        } else if (l.size() == 4) {
            this.ref1 = l.get(0);
            this.adj = l.get(1);
            this.degree = l.get(2);
            this.ref2 = l.get(3);
        } else {
            this.ref1 = l.get(0);
            this.adj = l.get(1);
            this.ref2 = l.get(2);
            this.degree = l.get(3);
            this.compTarget = l.get(4);
            this.ref3 = l.get(5);
        }
    }

    @Override
    public String getRef() {
        return ref1;
    }

    @Override
    public String toString() {
        if (ref3 != null) {
            return "property(" +ref1+","+adj+","+ref2+","+degree+","+compTarget+","+ref3+")";
        } else if (ref2 != null) {
            return "property(" +ref1+","+adj+","+degree+","+ref2+")";
        } else {
            return "property(" +ref1+","+adj+","+degree+")";
        }
    }

    public String getAdj() {
        return adj;
    }

    public String getCompTarget() {
        return compTarget;
    }

    public String getDegree() {
        return degree;
    }

    public String getRef1() {
        return ref1;
    }

    public String getRef2() {
        return ref2;
    }

    public String getRef3() {
        return ref3;
    }

    @Override
    public String getType() {
        return "PROP";
    }


}
